@@ -51,6 +51,8 @@ extern "C" {
 #define EXPORT
 #endif
 
+#define _celt_check_int(x) (((void)((x) == (int)0)), (int)(x))
+
 /* Error codes */
 /** No error */
 #define CELT_OK                0
@@ -62,8 +64,17 @@ extern "C" {
 #define CELT_INTERNAL_ERROR   -3
 /** The data passed (e.g. compressed data to decoder) is corrupted */
 #define CELT_CORRUPTED_DATA   -4
+/** Invalid/unsupported request number */
+#define CELT_UNIMPLEMENTED    -5
 
 /* Requests */
+#define CELT_SET_COMPLEXITY_REQUEST    2
+/** Controls the complexity from 0-10 (int) */
+#define CELT_SET_COMPLEXITY(x) CELT_SET_COMPLEXITY_REQUEST, _celt_check_int(x)
+#define CELT_SET_LTP_REQUEST    3
+/** Activate or deactivate the use of the long term predictor (PITCH) from 0 or 1 (int) */
+#define CELT_SET_LTP(x) CELT_SET_LTP_REQUEST, _celt_check_int(x)
+
 /** GET the frame size used in the current mode */
 #define CELT_GET_FRAME_SIZE   1000
 /** GET the lookahead used in the current mode */
@@ -101,7 +112,7 @@ typedef struct CELTMode CELTMode;
 /** Creates a new mode struct. This will be passed to an encoder or decoder.
     The mode MUST NOT BE DESTROYED until the encoders and decoders that use it
     are destroyed as well.
- @param Fs Sampling rate (32000 to 64000 Hz)
+ @param Fs Sampling rate (32000 to 96000 Hz)
  @param channels Number of channels
  @param frame_size Number of samples (per channel) to encode in each packet (64 - 256)
  @param lookahead Extra latency (in samples per channel) in addition to the frame size (between 32 and frame_size). The larger that value, the better the quality (at the expense of latency)
@@ -119,7 +130,6 @@ EXPORT void celt_mode_destroy(CELTMode *mode);
 /** Query information from a mode */
 EXPORT int celt_mode_info(const CELTMode *mode, int request, celt_int32_t *value);
 
-
 /* Encoder stuff */
 
 
@@ -138,18 +148,46 @@ EXPORT void celt_encoder_destroy(CELTEncoder *st);
 
 /** Encodes a frame of audio.
  @param st Encoder state
- @param pcm PCM audio in signed 16-bit format (native endian). There must be 
-            exactly frame_size samples per channel. The input data is 
-            overwritten by a copy of what the remote decoder would decode.
+ @param pcm PCM audio in signed float format. There must be 
+ *          exactly frame_size samples per channel. The input data is 
+ *          overwritten by a copy of what the remote decoder would decode.
+ @param optional_synthesis If not NULL, the encoder copies the audio signal that
+ *                         the decoder would decode. It is the same as calling the
+ *                         decoder on the compressed data, just faster.
  @param compressed The compressed data is written here
  @param nbCompressedBytes Number of bytes to use for compressing the frame
-                          (can change from one frame to another)
+ *                        (can change from one frame to another)
  @return Number of bytes written to "compressed". Should be the same as 
-         "nbCompressedBytes" unless the stream is VBR. If negative, an error
-         has occured (see error codes). It is IMPORTANT that the length returned
-         be somehow transmitted to the decoder. Otherwise, no decoding is possible.
+ *       "nbCompressedBytes" unless the stream is VBR. If negative, an error
+ *       has occured (see error codes). It is IMPORTANT that the length returned
+ *       be somehow transmitted to the decoder. Otherwise, no decoding is possible.
 */
-EXPORT int celt_encode(CELTEncoder *st, celt_int16_t *pcm, unsigned char *compressed, int nbCompressedBytes);
+EXPORT int celt_encode_float(CELTEncoder *st, const float *pcm, float *optional_synthesis, unsigned char *compressed, int nbCompressedBytes);
+/** Encodes a frame of audio.
+ @param st Encoder state
+ @param pcm PCM audio in signed 16-bit format (native endian). There must be 
+ *          exactly frame_size samples per channel. The input data is 
+ *          overwritten by a copy of what the remote decoder would decode.
+ @param optional_synthesis If not NULL, the encoder copies the audio signal that
+ *                         the decoder would decode. It is the same as calling the
+ *                         decoder on the compressed data, just faster.
+ @param compressed The compressed data is written here
+ @param nbCompressedBytes Number of bytes to use for compressing the frame
+ *                        (can change from one frame to another)
+ @return Number of bytes written to "compressed". Should be the same as 
+ *       "nbCompressedBytes" unless the stream is VBR. If negative, an error
+ *       has occured (see error codes). It is IMPORTANT that the length returned
+ *       be somehow transmitted to the decoder. Otherwise, no decoding is possible.
+ */
+EXPORT int celt_encode(CELTEncoder *st, const celt_int16_t *pcm, celt_int16_t *optional_synthesis, unsigned char *compressed, int nbCompressedBytes);
+
+/** Query and set encoder parameters 
+ @param st Encoder state
+ @param request Parameter to change or query
+ @param value Pointer to a 32-bit int value
+ @return Error code
+*/
+EXPORT int celt_encoder_ctl(CELTEncoder * st, int request, ...);
 
 /* Decoder stuff */
 
@@ -173,9 +211,19 @@ EXPORT void celt_decoder_destroy(CELTDecoder *st);
  @param len Number of bytes to read from "data". This MUST be exactly the number
             of bytes returned by the encoder. Using a larger value WILL NOT WORK.
  @param pcm One frame (frame_size samples per channel) of decoded PCM will be
-            returned here. 
+            returned here in float format. 
  @return Error code.
    */
+EXPORT int celt_decode_float(CELTDecoder *st, unsigned char *data, int len, float *pcm);
+/** Decodes a frame of audio.
+ @param st Decoder state
+ @param data Compressed data produced by an encoder
+ @param len Number of bytes to read from "data". This MUST be exactly the number
+            of bytes returned by the encoder. Using a larger value WILL NOT WORK.
+ @param pcm One frame (frame_size samples per channel) of decoded PCM will be
+            returned here in 16-bit PCM format (native endian). 
+ @return Error code.
+ */
 EXPORT int celt_decode(CELTDecoder *st, unsigned char *data, int len, celt_int16_t *pcm);
 
 /*  @} */
