@@ -69,7 +69,7 @@ int celt_mode_info(const CELTMode *mode, int request, celt_int32_t *value)
          *value = CELT_BITSTREAM_VERSION;
          break;
       default:
-         return CELT_BAD_ARG;
+         return CELT_UNIMPLEMENTED;
    }
    return CELT_OK;
 }
@@ -104,37 +104,44 @@ static const celt_int16_t pitch_freq[PBANDS+1] ={0, 345, 689, 1034, 1378, 2067,
 int BITALLOC_SIZE;
 int *band_allocation;
 #else
-#define BITALLOC_SIZE 10
+#define BITALLOC_SIZE 12
 static const int band_allocation[BARK_BANDS*BITALLOC_SIZE] = 
-   {  2,  2,  1,  1,  2,  2,  1,  1,  1,  1,  1,  1,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
+   {  4,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
+      2,  2,  1,  1,  2,  2,  1,  1,  1,  1,  1,  1,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
       2,  2,  2,  1,  2,  2,  2,  2,  2,  1,  2,  2,  4,  5,  7,  7,  7,  5,  4,  0,  0,  0,  0,  0,  0,
       2,  2,  2,  2,  3,  2,  2,  2,  2,  2,  3,  3,  5,  6,  8,  8,  8,  6,  5,  4,  0,  0,  0,  0,  0,
       3,  2,  2,  2,  3,  3,  2,  3,  2,  3,  4,  4,  6,  7,  9,  9,  9,  7,  6,  5,  5,  5,  0,  0,  0,
       3,  3,  2,  2,  3,  3,  3,  3,  3,  4,  4,  5,  7,  9, 10, 10, 10,  9,  6,  5,  5,  5,  5,  1,  0,
       4,  3,  3,  3,  3,  3,  3,  3,  4,  4,  6,  7,  7,  9, 11, 10, 10,  9,  9,  8, 11, 10, 10,  1,  1,
-      5,  5,  4,  4,  5,  5,  5,  5,  6,  6,  8,  8, 10, 12, 15, 15, 13, 12, 12, 12, 18, 18, 16, 10,  1,
-      6,  6,  6,  6,  6,  6,  7,  7,  9,  9, 11, 12, 13, 18, 22, 23, 24, 25, 28, 30, 35, 35, 35, 35, 15,
+      5,  5,  5,  5,  5,  5,  5,  6,  6,  6,  8,  8, 10, 12, 12, 11, 11, 17, 12, 15, 15, 20, 18, 10,  1,
+      8,  7,  7,  7,  7,  7,  8,  8,  9, 10, 11, 12, 14, 17, 18, 21, 22, 27, 29, 39, 37, 38, 40, 35,  1,
       7,  7,  7,  7,  7,  7, 10, 10, 10, 13, 14, 18, 20, 24, 28, 32, 32, 35, 38, 38, 42, 50, 59, 54, 31,
       8,  8,  8,  8,  8,  9, 10, 12, 14, 20, 22, 25, 28, 30, 35, 42, 46, 50, 55, 60, 62, 62, 62, 62, 62,
-};
+     12, 12, 12, 12, 12, 13, 15, 18, 22, 30, 32, 35, 40, 45, 55, 62, 66, 70, 85, 90, 92, 92, 92, 92, 92,
+   };
 #endif
 
 static celt_int16_t *compute_ebands(celt_int32_t Fs, int frame_size, int *nbEBands)
 {
    celt_int16_t *eBands;
-   int i, res, min_width, lin, low, high;
+   int i, res, min_width, lin, low, high, nBark;
    res = (Fs+frame_size)/(2*frame_size);
    min_width = MIN_BINS*res;
    /*printf ("min_width = %d\n", min_width);*/
 
+   /* Find the number of critical bands supported by our sampling rate */
+   for (nBark=1;nBark<BARK_BANDS;nBark++)
+    if (bark_freq[nBark+1]*2 >= Fs)
+       break;
+
    /* Find where the linear part ends (i.e. where the spacing is more than min_width */
-   for (lin=0;lin<BARK_BANDS;lin++)
+   for (lin=0;lin<nBark;lin++)
       if (bark_freq[lin+1]-bark_freq[lin] >= min_width)
          break;
    
    /*printf ("lin = %d (%d Hz)\n", lin, bark_freq[lin]);*/
    low = ((bark_freq[lin]/res)+(MIN_BINS-1))/MIN_BINS;
-   high = BARK_BANDS-lin;
+   high = nBark-lin;
    *nbEBands = low+high;
    eBands = celt_alloc(sizeof(celt_int16_t)*(*nbEBands+2));
    
@@ -148,7 +155,7 @@ static celt_int16_t *compute_ebands(celt_int32_t Fs, int frame_size, int *nbEBan
    for (i=0;i<*nbEBands;i++)
       if (eBands[i] < MIN_BINS*i)
          eBands[i] = MIN_BINS*i;
-   eBands[*nbEBands] = (bark_freq[BARK_BANDS]+res/2)/res;
+   eBands[*nbEBands] = (bark_freq[nBark]+res/2)/res;
    eBands[*nbEBands+1] = frame_size;
    if (eBands[*nbEBands] > eBands[*nbEBands+1])
       eBands[*nbEBands] = eBands[*nbEBands+1];
@@ -156,7 +163,8 @@ static celt_int16_t *compute_ebands(celt_int32_t Fs, int frame_size, int *nbEBan
    /* FIXME: Remove last band if too small */
    /*for (i=0;i<*nbEBands+2;i++)
       printf("%d ", eBands[i]);
-   printf ("\n");*/
+   printf ("\n");
+   exit(1);*/
    return eBands;
 }
 
@@ -198,24 +206,29 @@ static void compute_pbands(CELTMode *mode, int res)
 
 static void compute_allocation_table(CELTMode *mode, int res)
 {
-   int i, j, eband;
-   celt_int16_t *allocVectors;
-   
+   int i, j, eband, nBark;
+   celt_int16_t *allocVectors, *allocEnergy;
+   const int C = CHANNELS(mode);
+
+   /* Find the number of critical bands supported by our sampling rate */
+   for (nBark=1;nBark<BARK_BANDS;nBark++)
+    if (bark_freq[nBark+1]*2 >= mode->Fs)
+       break;
+
    mode->nbAllocVectors = BITALLOC_SIZE;
    allocVectors = celt_alloc(sizeof(celt_int16_t)*(BITALLOC_SIZE*mode->nbEBands));
+   allocEnergy = celt_alloc(sizeof(celt_int16_t)*(mode->nbAllocVectors*(mode->nbEBands+1)));
+   /* Compute per-codec-band allocation from per-critical-band matrix */
    for (i=0;i<BITALLOC_SIZE;i++)
    {
       eband = 0;
-      for (j=0;j<BARK_BANDS;j++)
+      for (j=0;j<nBark;j++)
       {
          int edge, low;
          celt_int32_t alloc;
          edge = mode->eBands[eband+1]*res;
          alloc = band_allocation[i*BARK_BANDS+j];
-         if (mode->nbChannels == 2)
-            alloc = alloc*3*mode->mdctSize/512;
-         else
-            alloc = alloc*mode->mdctSize/256;
+         alloc = alloc*C*mode->mdctSize/256;
          if (edge < bark_freq[j+1])
          {
             int num, den;
@@ -230,49 +243,37 @@ static void compute_allocation_table(CELTMode *mode, int res)
          }
       }
    }
-   /*for (i=0;i<BITALLOC_SIZE;i++)
+   /* Compute fine energy resolution and update the pulse allocation table to subtract that */
+   for (i=0;i<mode->nbAllocVectors;i++)
    {
+      int sum = 0;
       for (j=0;j<mode->nbEBands;j++)
-         printf ("%2d ", allocVectors[i*mode->nbEBands+j]);
-      printf ("\n");
-   }*/
+      {
+         int ebits;
+         int min_bits=0;
+         if (allocVectors[i*mode->nbEBands+j] > 0)
+            min_bits = 1;
+         ebits = IMAX(min_bits , allocVectors[i*mode->nbEBands+j] / (C*(mode->eBands[j+1]-mode->eBands[j])));
+         if (ebits>7)
+            ebits=7;
+         /* The bits used for fine allocation can't be used for pulses */
+         /* However, we give two "free" bits to all modes to compensate for the fact that some energy
+            resolution is needed regardless of the frame size. */
+         if (ebits>1)
+            allocVectors[i*mode->nbEBands+j] -= C*(ebits-2);
+         if (allocVectors[i*mode->nbEBands+j] < 0)
+            allocVectors[i*mode->nbEBands+j] = 0;
+         sum += ebits;
+         allocEnergy[i*(mode->nbEBands+1)+j] = ebits;
+      }
+      allocEnergy[i*(mode->nbEBands+1)+mode->nbEBands] = sum;
+   }
+   mode->energy_alloc = allocEnergy;
    mode->allocVectors = allocVectors;
 }
 
 #endif /* STATIC_MODES */
 
-static void compute_energy_allocation_table(CELTMode *mode)
-{
-   int i, j;
-   celt_int16_t *alloc;
-   
-   alloc = celt_alloc(sizeof(celt_int16_t)*(mode->nbAllocVectors*(mode->nbEBands+1)));
-   for (i=0;i<mode->nbAllocVectors;i++)
-   {
-      int sum = 0;
-      int min_bits = 1;
-      if (mode->allocVectors[i*mode->nbEBands]>12)
-         min_bits = 2;
-      if (mode->allocVectors[i*mode->nbEBands]>24)
-         min_bits = 3;
-      for (j=0;j<mode->nbEBands;j++)
-      {
-         alloc[i*(mode->nbEBands+1)+j] = mode->allocVectors[i*mode->nbEBands+j]
-                                         / (mode->eBands[j+1]-mode->eBands[j]-1);
-         if (alloc[i*(mode->nbEBands+1)+j]<min_bits)
-            alloc[i*(mode->nbEBands+1)+j] = min_bits;
-         if (alloc[i*(mode->nbEBands+1)+j]>7)
-            alloc[i*(mode->nbEBands+1)+j] = 7;
-         sum += alloc[i*(mode->nbEBands+1)+j];
-         /*printf ("%d ", alloc[i*(mode->nbEBands+1)+j]);*/
-         /*printf ("%f ", mode->allocVectors[i*mode->nbEBands+j]*1.f/(mode->eBands[j+1]-mode->eBands[j]-1));*/
-      }
-      alloc[i*(mode->nbEBands+1)+mode->nbEBands] = sum;
-      /*printf ("\n");*/
-   }
-   mode->energy_alloc = alloc;
-}
-
 CELTMode *celt_mode_create(celt_int32_t Fs, int channels, int frame_size, int *error)
 {
    int i;
@@ -293,8 +294,7 @@ CELTMode *celt_mode_create(celt_int32_t Fs, int channels, int frame_size, int *e
    {
       if (Fs == static_mode_list[i]->Fs &&
           channels == static_mode_list[i]->nbChannels &&
-          frame_size == static_mode_list[i]->mdctSize &&
-          lookahead == static_mode_list[i]->overlap)
+          frame_size == static_mode_list[i]->mdctSize)
       {
          m = static_mode_list[i];
          break;
@@ -317,9 +317,9 @@ CELTMode *celt_mode_create(celt_int32_t Fs, int channels, int frame_size, int *e
 
    /* The good thing here is that permutation of the arguments will automatically be invalid */
    
-   if (Fs < 32000 || Fs > 64000)
+   if (Fs < 32000 || Fs > 96000)
    {
-      celt_warning("Sampling rate must be between 32 kHz and 64 kHz");
+      celt_warning("Sampling rate must be between 32 kHz and 96 kHz");
       if (error)
          *error = CELT_BAD_ARG;
       return NULL;
@@ -333,7 +333,7 @@ CELTMode *celt_mode_create(celt_int32_t Fs, int channels, int frame_size, int *e
    }
    if (frame_size < 64 || frame_size > 512 || frame_size%2!=0)
    {
-      celt_warning("Only even frame sizes between 64 and 512 are supported");
+      celt_warning("Only even frame sizes from 64 to 512 are supported");
       if (error)
          *error = CELT_BAD_ARG;
       return NULL;
@@ -347,24 +347,35 @@ CELTMode *celt_mode_create(celt_int32_t Fs, int channels, int frame_size, int *e
    mode->eBands = compute_ebands(Fs, frame_size, &mode->nbEBands);
    compute_pbands(mode, res);
    mode->ePredCoef = QCONST16(.8f,15);
-   
-   if (frame_size <= 64)
+
+   if (frame_size > 384 && (frame_size%8)==0)
    {
-      mode->nbShortMdcts = 1;
-   } else if (frame_size <= 256)
+     mode->nbShortMdcts = 4;
+   } else if (frame_size > 384 && (frame_size%10)==0)
    {
-      mode->nbShortMdcts = 2;
-   } else if (frame_size <= 384)
+     mode->nbShortMdcts = 5;
+   } else if (frame_size > 256 && (frame_size%6)==0)
    {
-      mode->nbShortMdcts = 3;
-   } else {
-      mode->nbShortMdcts = 4;
+     mode->nbShortMdcts = 3;
+   } else if (frame_size > 256 && (frame_size%8)==0)
+   {
+     mode->nbShortMdcts = 4;
+   } else if (frame_size > 64 && (frame_size%4)==0)
+   {
+     mode->nbShortMdcts = 2;
+   } else if (frame_size > 128 && (frame_size%6)==0)
+   {
+     mode->nbShortMdcts = 3;
+   } else
+   {
+     mode->nbShortMdcts = 1;
    }
+
    if (mode->nbShortMdcts > 1)
-      mode->overlap = frame_size/mode->nbShortMdcts;
+      mode->overlap = ((frame_size/mode->nbShortMdcts)>>2)<<2; /* Overlap must be divisible by 4 */
    else
-      mode->overlap = frame_size/2;
-   
+      mode->overlap = (frame_size>>3)<<2;
+
    compute_allocation_table(mode, res);
    /*printf ("%d bands\n", mode->nbEBands);*/
    
@@ -380,8 +391,9 @@ CELTMode *celt_mode_create(celt_int32_t Fs, int channels, int frame_size, int *e
    mode->window = window;
 
    mode->bits = (const celt_int16_t **)compute_alloc_cache(mode, 1);
+   if (mode->nbChannels>=2)
+      mode->bits_stereo = (const celt_int16_t **)compute_alloc_cache(mode, mode->nbChannels);
 
-   mode->bits_stereo = NULL;
 #ifndef SHORTCUTS
    psydecay_init(&mode->psy, MAX_PERIOD/2, mode->Fs);
 #endif
@@ -397,10 +409,6 @@ CELTMode *celt_mode_create(celt_int32_t Fs, int channels, int frame_size, int *e
    mode->shortWindow = mode->window;
 
    mode->prob = quant_prob_alloc(mode);
-   compute_energy_allocation_table(mode);
-   
-   if (mode->nbChannels>=2)
-      mode->bits_stereo = (const celt_int16_t **)compute_alloc_cache(mode, mode->nbChannels);
 
    if (error)
       *error = CELT_OK;
@@ -421,12 +429,25 @@ void celt_mode_destroy(CELTMode *mode)
       }
    }
    celt_free((int**)mode->bits);
+   if (mode->bits_stereo != NULL)
+   {
+      for (i=0;i<mode->nbEBands;i++)
+      {
+         if (mode->bits_stereo[i] != prevPtr)
+         {
+            prevPtr = mode->bits_stereo[i];
+            celt_free((int*)mode->bits_stereo[i]);
+         }
+      }
+      celt_free((int**)mode->bits_stereo);
+   }
    if (check_mode(mode) != CELT_OK)
       return;
    celt_free((int*)mode->eBands);
    celt_free((int*)mode->pBands);
    celt_free((int*)mode->allocVectors);
-   
+   celt_free((celt_int16_t *)mode->energy_alloc);
+
    celt_free((celt_word16_t*)mode->window);
 
    mode->marker_start = MODEFREED;
@@ -436,9 +457,9 @@ void celt_mode_destroy(CELTMode *mode)
 #endif
 #endif
    mdct_clear(&mode->mdct);
+   mdct_clear(&mode->shortMdct);
    pitch_state_free(mode->fft);
    quant_prob_free(mode->prob);
-   celt_free((celt_int16_t *)mode->energy_alloc);
    celt_free((CELTMode *)mode);
 }
 
