@@ -249,7 +249,7 @@ FILE *out_file_open(char *outFile, int rate, int *channels)
 
 void usage(void)
 {
-   printf ("Usage: celtdec [options] input_file.spx [output_file]\n");
+   printf ("Usage: celtdec [options] input_file.oga [output_file]\n");
    printf ("\n");
    printf ("Decodes a CELT file and produce a WAV file or raw file\n");
    printf ("\n");
@@ -298,7 +298,7 @@ static CELTDecoder *process_header(ogg_packet *op, celt_int32_t enh_enabled, cel
       fprintf (stderr, "Unsupported number of channels: %d\n", header.nb_channels);
       return NULL;
    }
-   *mode = celt_mode_create(header.sample_rate, header.nb_channels, header.frame_size, header.overlap, NULL);
+   *mode = celt_mode_create(header.sample_rate, header.nb_channels, header.frame_size, NULL);
    if (*mode == NULL)
    {
       fprintf (stderr, "Mode initialization failed.\n");
